 /* Monte Carlo experiment */
new;
closeall;

/*Output file */
output file="Simu-MC1.out" reset;
output off;

/* True value for beta */
beta=2;

/*Directions*/
q=(-1)~1;

/* Sample sizes*/
matn=100~500~1000~2500;

/* Number of simulations */
nsimu=5000;


/* Grid for the interior test, betafront is the upper point on the frontier of the set */
Exv=sqrt(3)/2*(2*pdfn(0));
betafront=beta+Exv/2;
grille=seqa(0.01,0.01,200);
pts=-rev(grille)|0|grille;
betatest=pts'.*(betafront-beta)+beta;
nbpttest=rows(pts);

/* Level*/
alpha=0.05;

/* Initialisation random generator*/
rndseed 8769;

/* For saving the estimation results and the test results */
resint=((beta-Exv/2)|(beta+Exv/2));
resqt={};
restest=pts;




/* Loop across sample sizes */
rn=1;
do until rn>cols(matn);
n=matn[rn];

matdel={};


/* nsimu Simulations */
ns=1;
do until ns>nsimu;

/* DGP */
eps=rndu(n,1)-0.5;
x=(rndu(n,1)*2-1)*sqrt(3);
y=x*beta+eps;

v=rndu(n,1).*1.5-0.5;
v=rndn(n,1);

yinf=y+v.*(v.<0);
ysup=y+v.*(v.>=0);

/* Estimation of the support fonction for the two directions*/
sigmahat=inv((x'*x)/n);
zq=x*sigmahat*q;
wq=yinf+(ysup-yinf).*(zq.>0);
betaq=meanc((x*sigmahat).*wq);
deltaest=meanc(zq.*wq);


/* Estimation of the variance of the estimated support function*/
epsq=wq-x.*betaq';
zqeq=zq.*epsq;
vdelqn=(zqeq)'*zqeq/n; /* Variance of deltaq */
vq=vdelqn[1,1]|vdelqn[2,2]; /* je garde que pour -1 et +1 */


/***********Interior test************/
tinfq=(deltaest-q'.*betatest);
qn=minindc(tinfq);/* For each point tested, there is an estimated "worst" direction qn*/
tnqn=minc(tinfq);



/*Test statistic*/
xin=sqrt(n)*tnqn./sqrt(vq[qn]);

/*Rejection of the test*/
reject=cdfn(xin).<alpha;

/*Results saved*/
matdel=matdel|(betaq'~reject');

ns=ns+1;
endo;


/* We save the results: estimation of the interval and the interior test */
resint=resint~((meanc(matdel[.,1])~stdc(matdel[.,1]))|(meanc(matdel[.,2])~stdc(matdel[.,2])));
dmun=sortc(matdel[.,1],1);
dun=sortc(matdel[.,2],1);
resqt=resqt|((dmun[int(nsimu*0.25)])~(dmun[int(nsimu*0.5)])~(dmun[int(nsimu*0.75)])~(dun[int(nsimu*0.25)])~(dun[int(nsimu*0.5)])~(dun[int(nsimu*0.75)]));
restest=restest~(100*meanc(matdel[.,seqa(3,1,nbpttest)']));


rn=rn+1;
endo;

/* Results are displayed */
format /rd 10,4;

masque=ones(1,1+2*cols(matn));
fmt= (" *.*lf "~0~4)|("& *.*lf "~0~4).*.ones(2*cols(matn)-1,1)|("& *.*lf \\\\"~0~4);

output on;
"Estimation of the lower and upper points of the interval";"";
call printfm(resint,masque,fmt);
resqt;

output off;

masque=ones(1,cols(matn)+1);
fmt= ("*.*lf"~0~2)|("&*.*lf"~0~1).*.ones(cols(matn)-1,1)|("&*.*lf"~0~1);


output on;
"";"";"Interior Test";"";
call printfm(restest,masque,fmt);
"";
output off;
