 /* Monte Carlo experiment - second example */
new;
closeall;


/* Value of pi0 */
pi0=0.5;


/* Output file */
nomfic="Simu-MC2-pi0="$+ftocv(pi0*100,2,0)$+".out";
output file=^nomfic reset;
"pi0=" pi0;
output off;



/* Truve value*/
beta=2;

/* Directions */
q=-1~1;

/* Sample sizes*/
matn=100~500~1000~2500;

/*Number of simulations*/
nsimu=1000;

/*Points of the interval to be tested*/
Exv=sqrt(3)/2*(2*pdfn(0));
Eabsv=(2*pdfn(1)-2*cdfn(-1)+1);
betafront=beta+Exv/2;
grille=seqa(0.01,0.01,200);
/* calculation of the frontier point with z to check the size properties of the interior test*/
betafpi=(2.2534)*(pi0==0.5)+(pi0==0.1)*2.0866;
ptpi=(betafpi-beta)/(betafront-beta);


pts=ptpi|0|grille;
betatest=pts'.*(betafront-beta)+beta;
nbpttest=rows(pts);


/* Level of the test */
alpha=0.05;

/* Initialisation of the random generator if needed
rndseed 8769;*/

/* Variable for saving the estimation results and the test results */
resint=((2-Exv/2)|(2+Exv/2));
restest=pts;
ressargan={};


/* Loop */
rn=1;
do until rn>cols(matn);
n=matn[rn];

matdel={};
ns=1;
do until ns>nsimu;

/* DGP */
eps=rndu(n,1)-0.5;
x=(rndu(n,1)*2-1)*sqrt(3);
y=x*beta+eps;

v=rndn(n,1);



yinf=y+v.*(v.<0);
ysup=y+v.*(v.>=0);

/* Supernumerary instrument */

tir1=rndu(n,1);
test=(tir1.>(pi0));
z=test.*v+(1-test).*rndn(n,1);




/* Decomposition zf, zh */
zf=x;
zh=z-(meanc(z.*x)/meanc(x.*x))*x;
expl=zf~zh;
sigmahat=inv(expl'*expl/n);
ztq=expl*sigmahat;

/* Estimation of delta(+1,lambda) */
nbq=500;
lambda=tan(seqa(-Pi/2+Pi/2/nbq,Pi/(2*nbq),(2*nbq-1)));
q=ones(2*nbq-1,1)~lambda;
zq=ztq*q';
wq=yinf+(ysup-yinf).*(zq.>0);
deltaest=meanc(zq.*wq);
indl=minindc(deltaest); /* The lambda which minimizes delta(1,lambda) */
betaq=(meanc(zf.*wq)~meanc(zh.*wq))*sigmahat';
x11=betaq[.,1];
x21=betaq[.,2];
betaun=x11[indl]; /* beta(+1) */

/* Estimation of the Variance of delta(1) */
betaq=x11[indl]|x21[indl];
epsq=wq[.,indl]-expl*betaq;
zqeq=zq[.,indl].*epsq;
vdelun=(zqeq)'*zqeq/n; /* Variance de deltaq */


/* Estimation of delta(-1,lambda)  */
nbq=500;
lambda=tan(seqa(-Pi/2+Pi/2/nbq,Pi/(2*nbq),(2*nbq-1)));
q=(-ones(2*nbq-1,1))~lambda;
zq=ztq*q';
wq=yinf+(ysup-yinf).*(zq.>0);
deltaest=meanc(zq.*wq);
indl2=minindc(deltaest); /* The lambda which minimizes delta(-1,lambda) */
betaq=(meanc(zf.*wq)~meanc(zh.*wq))*sigmahat';
x12=betaq[.,1];
x22=betaq[.,2];
betamun=x12[indl2]; /* beta(-1) */



/* Estimation of the Variance of delta(-1) */
betaq=x12[indl2]|x22[indl2];
epsq=wq[.,indl2]-expl*betaq;
zqeq=zq[.,indl2].*epsq;
vdelmun=(zqeq)'*zqeq/n; /* Variance de deltaq */


/* The results for the interior test */
vq=vdelmun|vdelun;
deltaest=-betamun|betaun;

/* Interior test*/
q0=-1~1;
tinfq=(deltaest-q0'.*betatest);
qn=minindc(tinfq);/* The qn which minimizes Tn(q) for each point tested */
tnqn=minc(tinfq);



/*Test statistic */
xin=sqrt(n)*tnqn./sqrt(vq[qn]);

/* Decision of rejection */
reject=cdfn(xin).<alpha;



/************ Sargan Test ************/
/* delta(Pi/2) */
q=0|1;
zq=ztq*q;
wq=yinf+(ysup-yinf).*(zq.>0);
deltapisd=meanc(zq.*wq);
betapisd=sigmahat*(meanc(zf.*wq)|meanc(zh.*wq));
/* Variance of delta(Pi/2) */
epsq=wq-expl*betapisd;
zqeq=zq.*epsq;
vdelpisd=(zqeq)'*zqeq/n;

/* delta(3Pi/2) */
q=0|(-1);
zq=ztq*q;
wq=yinf+(ysup-yinf).*(zq.>0);
delta3pisd=meanc(zq.*wq);
beta3pisd=sigmahat*(meanc(zf.*wq)|meanc(zh.*wq));
/* Variance of delta(3Pi/2) */
epsq=wq-expl*beta3pisd;
zqeq=zq.*epsq;
vdel3pisd=(zqeq)'*zqeq/n;


/* test statistic for Sargan */
vsarg=vdelpisd|vdel3pisd;
tinfq=(deltapisd)|(delta3pisd);
qn=minindc(tinfq);/* Le qn qui acheve le minimum de Tn(q)*/
tnqn=minc(tinfq);
xisargan=sqrt(n)*tnqn./sqrt(vsarg[qn]);



/* Decision of rejection */
rejectsargan=cdfn(xisargan).<alpha;

/*********** End of Sargan Test **************/


/* We save the results for this simulation*/
matdel=matdel|(reject'~rejectsargan);


ns=ns+1;
endo;


/* Interior Test for this sample size */
restest=restest~(100*meanc(matdel[.,seqa(1,1,nbpttest)']));

/* Sargan Test for this sample size */
ressargan=ressargan~(100*meanc(matdel[.,cols(matdel)]));
rn=rn+1;/* next sample size */
endo;





/* Results are displayed */
format /rd 10,4;
masque=ones(1,cols(matn));
fmt= (" *.*lf "~0~4)|("& *.*lf "~0~4).*.ones(cols(matn)-2,1)|("& *.*lf \\\\"~0~4);

output on;
"Sargan Test ";"";

call printfm(ressargan,masque,fmt);
"";
output off;

masque=ones(1,cols(matn)+1);
fmt= (" *.*lf \\\%"~0~3)|("& *.*lf \\\%"~0~4).*.ones(cols(matn)-1,1)|("& *.*lf \\\%\\\\"~0~4);

output on;
"";"";"Interior Test";"";
call printfm(restest,masque,fmt);
